/*
 * $Workfile:: Example.c                                                      $
 * $Revision:: 4                                                              $
 *
 * $Author:: Buck Rogers                                                      $
 * $Modtime:: 29.10.1997 17:41 Uhr                                            $
 *
 * $History:: Example.c                                                       $
 * 
 * *****************  Version 4  *****************
 * User: Buck Rogers  Date: 29.10.1997   Time: 17:42 Uhr
 * Updated in $/BSNG/Plugins/BSNG SDK/Example Plugin
 * example plugin now makes use of the new 'errorText' variable
 * 
 * *****************  Version 3  *****************
 * User: Buck Rogers  Date: 08.10.1997   Time: 02:03 Uhr
 * Updated in $/BSNG/Plugins/BSNG SDK/Example Plugin
 * changed lf to cr in list generation so Mac compatible output is created
 * 
 * *****************  Version 2  *****************
 * User: Buck Rogers  Date: 05.10.1997   Time: 19:24 Uhr
 * Updated in $/BSNG/Plugins/BSNG SDK/Example Plugin
 * Added comments related to the new name, company and numCopies items in the
 * parameter block
 * 
 * *****************  Version 1  *****************
 * User: Buck Rogers  Date: 30.09.1997   Time: 18:31 Uhr
 * Created in $/BSNG/Plugins/BSNG SDK/Example Plugin
 * Adding subproject 'BSNG' to '$/'
 *
 * $NoKeywords::                                                              $
 */


#include <MixedMode.h>
#include <A4Stuff.h>

#include "standard utils.h"
#include "UltraU.h"
#include "BSNG API.h"


/*	don't touch this __procinfo definition or the calls from native code will crash the machine */

ProcInfoType __procinfo = kThinkCStackBased | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(BSNGParamBlockPtr)));


/*	function prototypes for ANSI C */

static void DoInit(BSNGParamBlockPtr inData);
static Boolean DoValidate(BSNGParamBlockPtr inData);
static void DoCalc(BSNGParamBlockPtr inData);
static void DoRandomCalc(BSNGParamBlockPtr inData);
static Boolean DoAddRandomsToList(BSNGParamBlockPtr inData);
static void DoItemHit(BSNGParamBlockPtr inData);
static void DoCleanup(BSNGParamBlockPtr inData);
void main(BSNGParamBlockPtr inData);


/*	here is where the fun starts :-) */

static void DoInit(BSNGParamBlockPtr inData)
{
	StringPtr		aStr = nil;
	
	
	/*	Initialize the Ultra Random Number generator (optional, use MacOS random functions otherwise) */
	
	Ultra_seed1 = inData->randSeed1;
	Ultra_seed2 = inData->randSeed2;
	Ultra_Init();
	
	/*	Give the BSNG App the informations if your plugin can do random calculations and if it can write
		to the serial number list */
	
	inData->wantsRandomButton = true;
	inData->canAddToSNList = true;
	
	/*	Initialize the EditFields with initial default values (optional, they are empty otherwise) */
	/*	if you create a name-based generator please make sure that you copy the name, company and/or numCopies */
	/*	values into the corrosponding fields */
	
	aStr = inData->itemValue[kItemValue1];
	BlockMoveData("\p12345678", aStr, 9);
	
	aStr = inData->itemValue[kItemValue2];
	BlockMoveData("\p23456789", aStr, 9);
	
	aStr = inData->itemValue[kItemValue3];
	BlockMoveData("\p34567890", aStr, 9);
	
	aStr = inData->itemValue[kItemValue4];
	aStr[0] = 0;
	
	/*	Tell the BSNG App if the initialisation was successful (it was in our case) */
	
	inData->errorText[0] = 0;
	inData->error = errExtNoErr;
	inData->errorInItem = 0;
}  /* DoInit */


static Boolean DoValidate(BSNGParamBlockPtr inData)
{
	StringPtr		num1Str = inData->itemValue[kItemValue1];
	StringPtr		num2Str = inData->itemValue[kItemValue2];
	StringPtr		num3Str = inData->itemValue[kItemValue3];
	
	Str31			inputStr	= "\pThe input for ";
	Str31			tooLongStr	= "\p is too long! ";
	Str31			emptyStr	= "\p is empty! ";
	Str63			pleaseStr	= "\p\rPlease enter a number between 1 and 99999999.";
	
	Boolean			error = false;
	
	
	/*	we do some simple checks here if the edit fields are not empty or not longer than 8 characters, additionally we could
		also check if the values are really only numbers, but I skipped that because the input filter for the edit fileds
		was set to integer anyway (defined in Constructor) */
	
	inData->errorText[0] = 0;
	
	if ((num1Str[0] < 1) || (num1Str[0] > 8))
	{
		/*	if something was wrong in the input tell the BSNG App what edit field contained wrong values */
		
		inData->errorInItem = kEditItem1;
		error = true;
		
		/*	build the errorText string (optional), leave errorText empty if you don't want to use this feature */
		
		myAppendPStr(inData->errorText, inputStr);
		myAppendPStr(inData->errorText, "\pNumber 1");
		
		if (num1Str[0] < 1)
		{
			myAppendPStr(inData->errorText, emptyStr);
		}
		else
		{
			myAppendPStr(inData->errorText, tooLongStr);
		}
		
		myAppendPStr(inData->errorText, pleaseStr);
	}
	else if ((num2Str[0] < 1) || (num2Str[0] > 8))
	{
		inData->errorInItem = kEditItem2;
		error = true;
		
		myAppendPStr(inData->errorText, inputStr);
		myAppendPStr(inData->errorText, "\pNumber 2");
		
		if (num2Str[0] < 1)
		{
			myAppendPStr(inData->errorText, emptyStr);
		}
		else
		{
			myAppendPStr(inData->errorText, tooLongStr);
		}
		
		myAppendPStr(inData->errorText, pleaseStr);
	}
	else if ((num3Str[0] < 1) || (num3Str[0] > 8))
	{
		inData->errorInItem = kEditItem3;
		error = true;
		
		myAppendPStr(inData->errorText, inputStr);
		myAppendPStr(inData->errorText, "\pNumber 3");
		
		if (num3Str[0] < 1)
		{
			myAppendPStr(inData->errorText, emptyStr);
		}
		else
		{
			myAppendPStr(inData->errorText, tooLongStr);
		}
		
		myAppendPStr(inData->errorText, pleaseStr);
	}
	
	if (error)
	{
		inData->error = errExtIncorrectValue;
		return (false);
	}
	
	inData->error = errExtNoErr;
	
	return (true);
}  /* DoValidate */


static void DoCalc(BSNGParamBlockPtr inData)
{
	StringPtr		num1Str = inData->itemValue[kItemValue1];
	StringPtr		num2Str = inData->itemValue[kItemValue2];
	StringPtr		num3Str = inData->itemValue[kItemValue3];
	StringPtr		addResultStr = inData->itemValue[kItemValue4];
	
	unsigned long	result = 0L;
	long			value = 0L;
	Boolean			error = false;
	short			i = 0;
	
	
	/*	very simply algorithm, just add the 3 numbers and set the result */
	
	StringToNum(num1Str, &value);
	result += value;
	
	StringToNum(num2Str, &value);
	result += value;
	
	StringToNum(num3Str, &value);
	result += value;
	
	NumToString(result, addResultStr);
	
	/*	put the result on the Clipboard so users can copy and paste the serial number into the app they want to register */
	
	addResultStr[addResultStr[0] + 1] = '\0';
	
	ZeroScrap();
	PutScrap(addResultStr[0], 'TEXT', &(addResultStr[1]));
}  /* DoCalc */


static void DoItemHit(BSNGParamBlockPtr inData)
{
	switch (inData->itemMessage)
	{
		case (300000):
		
			/*	the "About this plugin" button was pressed, do whatever you want, I just display a small Alert here */
		
			NoteAlert(1000, nil);
			break;
		
		default:
			break;
	}
}  /* DoItemHit */


static void DoRandomCalc(BSNGParamBlockPtr inData)
{
	StringPtr		num1Str = inData->itemValue[kItemValue1];
	StringPtr		num2Str = inData->itemValue[kItemValue2];
	StringPtr		num3Str = inData->itemValue[kItemValue3];
	
	
	/*	create your own random numbers as the random functions are not dependent on user input */
	
	num1Str[0] = 0;									/*	empty the string */
	myBaseBRandomPStr(num1Str, 10, 8);				/*	generate 8 random digits */
	
	num2Str[0] = 0;
	myBaseBRandomPStr(num2Str, 10, 8);
	
	num3Str[0] = 0;
	myBaseBRandomPStr(num3Str, 10, 8);
	
	/*	now we are prepared and can call our Calc algorithm */
	
	DoCalc(inData);
}  /* DoRandomCalc */


static Boolean DoAddRandomsToList(BSNGParamBlockPtr inData)
{
	StringPtr		num1Str = inData->itemValue[kItemValue1];
	StringPtr		num2Str = inData->itemValue[kItemValue2];
	StringPtr		num3Str = inData->itemValue[kItemValue3];
	StringPtr		addResultStr = inData->itemValue[kItemValue4];
	
	long			count = 0L;
	short			i = 0;
	OSErr			err = noErr;
	
	
	/*	write your header to the number list */
	
	count = 24L;
	err = FSWrite(inData->outputRefNum, &count, "BSNG SDK Example Plugin\r");
	
	/*	return false if an error occured, the creation of list will then be completly aborted */
	
	if (err != noErr)
	{
		return (false);
	}
	
	count = 24L;
	err = FSWrite(inData->outputRefNum, &count, "=======================\r");
	
	if (err != noErr)
	{
		return (false);
	}
	
	/*	create numOfListNumbers random serial numbers and write them to the list */
	/*	if you create a name-based generator please make sure that you create at least one number */
	/*	using the name, company and/or numCopies informations. You could create more name-based numbers */
	/*	by using your own name/company database */
	
	for (i = 0; i < inData->numOfListNumbers; i++)
	{
		DoRandomCalc(inData);
		
		count = (long) num1Str[0];
		err = FSWrite(inData->outputRefNum, &count, &(num1Str[1]));
		
		if (err != noErr)
		{
			return (false);
		}
	
		count = 3L;
		err = FSWrite(inData->outputRefNum, &count, " + ");
		
		if (err != noErr)
		{
			return (false);
		}
		
		count = (long) num2Str[0];
		err = FSWrite(inData->outputRefNum, &count, &(num2Str[1]));
		
		if (err != noErr)
		{
			return (false);
		}
	
		count = 3L;
		err = FSWrite(inData->outputRefNum, &count, " + ");
		
		if (err != noErr)
		{
			return (false);
		}
		
		count = (long) num3Str[0];
		err = FSWrite(inData->outputRefNum, &count, &(num3Str[1]));
		
		if (err != noErr)
		{
			return (false);
		}
	
		count = 3L;
		err = FSWrite(inData->outputRefNum, &count, " = ");
		
		if (err != noErr)
		{
			return (false);
		}
		
		count = (long) addResultStr[0];
		err = FSWrite(inData->outputRefNum, &count, &(addResultStr[1]));
		
		if (err != noErr)
		{
			return (false);
		}
	
		count = 1L;
		err = FSWrite(inData->outputRefNum, &count, "\r");
		
		if (err != noErr)
		{
			return (false);
		}
	}
	
	/* don't forget this last newline */
	
	count = 1L;
	err = FSWrite(inData->outputRefNum, &count, "\r");
	
	if (err != noErr)
	{
		return (false);
	}
	
	return (true);
}  /* DoAddRandomsToList */


static void DoCleanup(BSNGParamBlockPtr inData)
{
#pragma unused (inData)

	/*	we didn't allocate any memory in DoInit, so we can leave this empty, otherwise it would be a VERY good idea
		to deallocate/dispose etc. everything we allocated during our work. If you don't do that we have a nice memory leak */
	
}  /* DoCleanup */


void main(BSNGParamBlockPtr inData)
{
#if (!GENERATINGPOWERPC)
	EnterCodeResource();
#endif
	
	/*	the message dispatcher */
	
	switch(inData->theMessage)
	{
		case (msgExtInit):
			DoInit(inData);
			break;
		
		case (msgExtCalcHit):
		
			if (DoValidate(inData))
			{
				DoCalc(inData);
			}
			
			break;
		
		case (msgExtRandomHit):
			DoRandomCalc(inData);
			break;
		
		case (msgExtCreateRandom):
			
			/*	report to the BSNG App if your list entry was written ok or if we had an error */
			
			if (DoAddRandomsToList(inData))
			{
				inData->error = errExtNoErr;
			}
			else
			{
				inData->error = errExtWritingToList;
			}
			
			break;
		
		case (msgExtItemHit):
			DoItemHit(inData);
			break;
		
		case (msgExtCleanup):
			DoCleanup(inData);
			break;
		
		default:
			break;
	}
	
#if (!GENERATINGPOWERPC)
	ExitCodeResource();
#endif
}  /* main */
